'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var reactRedux = require('react-redux');
var immutable = require('immutable');
var cursor = require('immutable/contrib/cursor');
var reduxActions = require('redux-actions');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

function buildFastReducer(redcuerKey) {
    var _a;
    var NAME_SPACE = 'fast_' + redcuerKey;
    var REDUCER_KEY = redcuerKey;
    var types = {
        fastState: NAME_SPACE + '--state',
        fastUnmount: NAME_SPACE + '--unmount',
    };
    /**
     * 快速改变状态
     * @param keyPath immatable路径
     * @param cd 回调，改变状态的函数
     * @param meta action的meta
     */
    var setFastState = function (cb, meta) {
        return {
            type: types.fastState,
            payload: cb,
            meta: meta,
        };
    };
    var getFastState = function () { return function (dispatch, getState) {
        return cursor.from(getState()[REDUCER_KEY]);
    }; };
    var reduccerMap = (_a = {},
        _a[types.fastState] = function (state, action) {
            return state.updateIn(action.payload.keyPath, function (s) {
                if (s === void 0) { s = immutable.fromJS({}); }
                return action.payload.cb(s);
            });
        },
        _a[types.fastUnmount] = function (state, action) {
            return state.deleteIn(action.payload.keyPath);
        },
        _a);
    return {
        reduccerMap: reduccerMap,
        setFastState: setFastState,
        getFastState: getFastState,
    };
}

var _a;
var NAME_SPACE = 'fast';
var REDUCER_KEY = 'fast';
var types = {
    fastState: NAME_SPACE + '--state',
    fastUnmount: NAME_SPACE + '--unmount',
};
var getFastState = function (keyPath) { return function (dispatch, getState) {
    return cursor.from(getState()[REDUCER_KEY]).getIn(keyPath);
}; };
var unmount = function (keyPath) {
    return {
        type: types.fastUnmount,
        payload: {
            keyPath: keyPath,
        },
    };
};
var reduccer = reduxActions.handleActions((_a = {},
    _a[types.fastState] = function (state, action) {
        return state.updateIn(action.payload.keyPath, function (s) {
            if (s === void 0) { s = immutable.fromJS({}); }
            return action.payload.cb(s);
        });
    },
    _a[types.fastUnmount] = function (state, action) {
        return state.deleteIn(action.payload.keyPath);
    },
    _a), immutable.fromJS({}));

var ReduxFast = /** @class */ (function (_super) {
    __extends(ReduxFast, _super);
    function ReduxFast() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            didMount: false,
        };
        return _this;
    }
    ReduxFast.prototype.componentDidMount = function () {
        this.props.didMount && this.props.didMount();
        this.setState({ didMount: true });
    };
    ReduxFast.prototype.componentWillUnmount = function () {
        this.props.dispatch(unmount(this.props.keyPath));
    };
    ReduxFast.prototype.render = function () {
        var state = this.props.dispatch(getFastState(this.props.keyPath));
        if (!state) {
            return '';
        }
        return this.props.children(state);
    };
    return ReduxFast;
}(React.Component));
var index = reactRedux.connect(function () {
    return {};
}, function (dispatch) { return ({ dispatch: dispatch }); })(ReduxFast);

exports.buildFastReducer = buildFastReducer;
exports.default = index;
